;;  edivad-suite a simple suite of gimp scripts for simplify my life
;;  Copyright (C) 2008  Davide Giannella <edivad_@users.sourceforge.net>
;;
;;  This program is free software; you can redistribute it and/or modify
;;  it under the terms of the GNU General Public License as published by
;;  the Free Software Foundation; version 2 of the License.
;; 
;;  This program is distributed in the hope that it will be useful,
;;  but WITHOUT ANY WARRANTY; without even the implied warranty of
;;  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;  GNU General Public License for more details.
;;
;;  You should have received a copy of the GNU General Public License along
;;  with this program; if not, write to the Free Software Foundation, Inc.,
;;  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

;;(set! WM-PATTERNS '("Top Left" "Top Right" "Center" "Bottom Left" "Bottom Right" "Auris"))
(define WM-PATTERNS '("Top Left" "Top Right" "Center" "Bottom Left" "Bottom Right" "Auris"))

;;;;
;; create the top-left wm pattern
;; @param txt the layer with the text
(define (edivad-wm-tl txt)
  (let*
      ()
    (gimp-layer-set-offsets txt 0 0)
    )
  )

;;;;
;; create the top-right wm pattern
;; @param txt the layer with the text
(define (edivad-wm-tr txt)
  (let*
      (
       (txt-width (car (gimp-drawable-width txt)))
       (img-width (car (gimp-image-width (car (gimp-drawable-get-image txt)))))
       (x (- img-width txt-width))
       )
    (gimp-layer-set-offsets txt x 0)
    )
  )

;;;;
;; create the center wm pattern
;; @param txt the layer with the text
(define (edivad-wm-cn txt)
  (let*
      (
       (txt-width (car (gimp-drawable-width txt)))
       (txt-height (car (gimp-drawable-height txt)))
       (img-width (car (gimp-image-width (car (gimp-drawable-get-image txt)))))
       (img-height (car (gimp-image-height (car (gimp-drawable-get-image txt)))))
       (x (- (/ img-width 2) (/ txt-width 2)))
       (y (- (/ img-height 2) (/ txt-height 2)))
       )
    (gimp-layer-set-offsets txt x y)
    )
  )

;;;;
;; create the bottom left wm pattern
;; @param txt the layer with the text
(define (edivad-wm-bl txt)
  (let*
      (
       (txt-width (car (gimp-drawable-width txt)))
       (txt-height (car (gimp-drawable-height txt)))
       (img-width (car (gimp-image-width (car (gimp-drawable-get-image txt)))))
       (img-height (car (gimp-image-height (car (gimp-drawable-get-image txt)))))
       (x 0)
       (y (- img-height txt-height 2))
       )
    (gimp-layer-set-offsets txt x y)
    )
  )

;;;;
;; create the bottom left wm pattern
;; @param txt the layer with the text
(define (edivad-wm-br txt)
  (let*
      (
       (txt-width (car (gimp-drawable-width txt)))
       (txt-height (car (gimp-drawable-height txt)))
       (img-width (car (gimp-image-width (car (gimp-drawable-get-image txt)))))
       (img-height (car (gimp-image-height (car (gimp-drawable-get-image txt)))))
       (x (- img-width txt-width))
       (y (- img-height txt-height))
       )
    (gimp-layer-set-offsets txt x y)
    )
  )

;;;;
;; create the bottom left wm pattern
;; @param txt the layer with the text
;; @return the newly merged layer
(define (edivad-wm-auris txt)
  (let*
      (
       (txt-width (car (gimp-drawable-width txt)))
       (txt-height (car (gimp-drawable-height txt)))
       (img (car (gimp-drawable-get-image txt)))
       (img-width (car (gimp-image-width img)))
       (img-height (car (gimp-image-height img)))
       (txt-tl (car (gimp-layer-copy txt 0)))
       (txt-tr (car (gimp-layer-copy txt 0)))
       (txt-bl (car (gimp-layer-copy txt 0)))
       (txt-br (car (gimp-layer-copy txt 0)))
       (txt-cn txt)
       (merged txt)
       )
    ;adding duplicated layers to the image
    (gimp-image-add-layer img txt-tl -1)
    (gimp-image-add-layer img txt-tr -1)
    (gimp-image-add-layer img txt-bl -1)
    (gimp-image-add-layer img txt-br -1)
    
    ;positioning layers
    (edivad-wm-tl txt-tl)
    (edivad-wm-tr txt-tr)
    (edivad-wm-bl txt-bl)
    (edivad-wm-br txt-br)
    (edivad-wm-cn txt-cn)
    
    ;mergin down from the last created layer
    (set! merged (car (gimp-image-merge-down img txt-br 1)))
    (set! merged (car (gimp-image-merge-down img merged 1)))
    (set! merged (car (gimp-image-merge-down img merged 1)))
    (set! merged (car (gimp-image-merge-down img merged 1)))
    
    (list merged)
    )
  )

;;;;
;; create the final resulting pattern
;; @param patt the selected parttern
;; @param txt the layer with the watermark
;; @return the layer with the pattern specified
(define (create-pattern patt txt)
  (let*
      (
       (ret txt)
       )
    (cond
     ((= patt 0) (edivad-wm-tl txt)) ;;Top Left
     ((= patt 1) (edivad-wm-tr txt)) ;;Top Right
     ((= patt 2) (edivad-wm-cn txt)) ;;Center
     ((= patt 3) (edivad-wm-bl txt)) ;;Bottom Left
     ((= patt 4) (edivad-wm-br txt)) ;;Bottom Left
     ((= patt 5) (set! ret (car (edivad-wm-auris txt)))) ;;Auris
     )
    (list ret)
    )
  )

;;;;
;; Add a watermark to the current image
;; @param img SF-IMAGE the opened image
;; @param drw SF-DRAWABLE the acrive drawable
;; @param font SF-FONT the desired font
;; @param size SF-ADJUSTMENT the desired font size
;; @param text SF-STRING text to add
;; @param pattern SF-OPTION the pattern to apply for the watermark
;; @return display, image : the display and image just created
(define (edivad-watermark img drw font size text pattern)
  (let*
      (
       (fname (parse-name (car (gimp-image-get-filename img))))
       (fname (string-append fname "-wm.tiff"))
       (new-img (car (gimp-image-duplicate img)))
       (drw-new-img (car (gimp-image-get-active-drawable new-img)))
       (img-mode (car (gimp-drawable-is-rgb drw-new-img)))
       (display (car (gimp-display-new new-img)))
       (width (car (gimp-image-width new-img)))
       (height (car (gimp-image-height new-img)))
       (azimuth 135)
       (elevation 45)
       (depth 1)
       (layer-img 0)
       (fg 0)
       (layer-black 0)
       (text-layer 0)
       (patt-layer 0)
       )
    (gimp-image-set-filename new-img fname)
    (gimp-image-undo-group-start new-img)
    (if (= img-mode 0)
        (gimp-image-convert-rgb new-img)
        )
    (set! layer-img (car (gimp-image-flatten new-img)))
    
    ;;adding black layer
    (set! layer-black (car (gimp-layer-new new-img width height 1 "black layer" 100 0)))
    (gimp-image-add-layer new-img layer-black -1)
    (set! fg (car (gimp-context-get-foreground)))
    (gimp-context-set-foreground '(0 0 0))
    (gimp-drawable-fill layer-black 0)
    (gimp-context-set-foreground fg)

    ;;adding white text and blur it
    (set! fg (car (gimp-context-get-foreground)))
    (gimp-context-set-foreground '(255 255 255))
    (set! text-layer (car (gimp-text-fontname new-img -1 0 0 text -1 TRUE size 0 font)))
    (gimp-context-set-foreground fg)
    (plug-in-gauss 1 new-img text-layer 3 3 0)

    ;;create desired watermak pattern
    (set! patt-layer (car (create-pattern pattern text-layer)))
    
    ;;merging down the pattern layer. The just down one should be the black.
    (set! patt-layer (car (gimp-image-merge-down new-img patt-layer 1)))
    (gimp-drawable-set-name patt-layer "pattern")
    
    ;;creating the bump map
    (plug-in-bump-map 1 new-img layer-img patt-layer azimuth elevation depth 0 0 0 0 1 0 0)
    (gimp-drawable-set-visible patt-layer 0)
    (gimp-image-set-active-layer new-img layer-img)
    
    (gimp-image-undo-group-end new-img)
    (gimp-displays-flush)
    
    (list display new-img)
    )
  )

;;;;
;; Add a watermark to the current image
;; @param path SF-FILENAME the path to the image to process
;; @param font SF-FONT the desired font
;; @param size SF-ADJUSTMENT the desired font size
;; @param text SF-STRING text to add
;; @param pattern SF-OPTION the pattern to apply for the watermark
;; @return display, image : the display and image just created
(define (edivad-watermark-file path font size text pattern)
  (let*
      (
       (image (car (gimp-file-load RUN-NONINTERACTIVE path path))) ;load image
       (drawable (car (gimp-image-get-active-layer image))) ;get drawable
       (ret (edivad-watermark image drawable font size text pattern)) ; creating the watermark
       (display (car ret))
       (image-wm (car (cdr ret)))
       (file (car (gimp-image-get-filename image-wm)))
       )
    (gimp-image-flatten image-wm)
    (gimp-file-save 1 image-wm (car (gimp-image-get-active-drawable image-wm)) file file)
    (gimp-display-delete display)
    (gimp-image-delete image)
    )
  )

;;;;
;; Take the selected directory by pattern and add a watermark to every images
;; 
;; @param dir SF-DIRNAME the directory to parse
;; @param pattern SF-OPTION the pattern
;; @param dpi SF-VALUE the desired dpi 
(define (edivad-watermark-dir dir ft font size text pattern)
  (let*
      (
       (ptr (car (build-pattern dir (car (get-pattern ft)))))
       (filelist 0)
       (filename 0)
      )
    (set! filelist (cadr (file-glob ptr 1)))
    (while (not (null? filelist))
           (set! filename (car filelist))
           (edivad-watermark-file filename font size text pattern)
           (set! filelist (cdr filelist))
    )
  )
)

;;(edivad-watermark 1 2 "Sans" 30 "Your Watermark" 1)
(script-fu-register "edivad-watermark"
                    _"<Image>/Script-Fu/edivad-suite/Watermark"
                    "Take the active image and add a watermark"
                    "Davide Giannella"
                    "(C) Davide Giannella"
                    "March 2009"
                    ""
                    SF-IMAGE "Image" 0
                    SF-DRAWABLE "Drawable" 0
                    SF-FONT "Font" "Sans"
                    SF-ADJUSTMENT  "Font size" '(30 1 1000 1 10 0 1)
                    SF-STRING "Text" "Your watermark"
                    SF-OPTION "Pattern" WM-PATTERNS
)

;;(edivad-watermark-file "c:/3c.xcf" "Sans" 30 "Your watermark" 0)
(script-fu-register "edivad-watermark-file"
                    _"<Toolbox>/Xtns/Script-Fu/edivad-suite/Watermark/Single File"
                    "Take the selected image and add a watermark"
                    "Davide Giannella"
                    "(C) Davide Giannella"
                    "March 2009"
                    ""
                    SF-FILENAME "File path" ""
                    SF-FONT "Font" "Sans"
                    SF-ADJUSTMENT  "Font size" '(30 1 1000 1 10 0 1)
                    SF-STRING "Text" "Your watermark"
                    SF-OPTION "Pattern" WM-PATTERNS
)

(script-fu-register "edivad-watermark-dir"
                    _"<Toolbox>/Xtns/Script-Fu/edivad-suite/Watermark/Directory"
                    "Take the selected directory and convert all images matching pattern into jpg 72dpi"
                    "Davide Giannella"
                    "(C) Davide Giannella"
                    "March 2009"
                    ""
                    SF-DIRNAME "Directory" ""
                    SF-OPTION "File type" PATTERNS
                    SF-FONT "Font" "Sans"
                    SF-ADJUSTMENT  "Font size" '(30 1 1000 1 10 0 1)
                    SF-STRING "Text" "Your watermark"
                    SF-OPTION "Pattern" WM-PATTERNS
)