;;  edivad-suite a simple suite of gimp scripts for simplify my life
;;  Copyright (C) 2008  Davide Giannella <edivad_@users.sourceforge.net>
;;
;;  This program is free software; you can redistribute it and/or modify
;;  it under the terms of the GNU General Public License as published by
;;  the Free Software Foundation; version 2 of the License.
;; 
;;  This program is distributed in the hope that it will be useful,
;;  but WITHOUT ANY WARRANTY; without even the implied warranty of
;;  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;  GNU General Public License for more details.
;;
;;  You should have received a copy of the GNU General Public License along
;;  with this program; if not, write to the Free Software Foundation, Inc.,
;;  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

;;;;
;; Take the active image and convert it into jpg 72dpi
;; 
;; @param img SF-IMAGE the opened image
;; @param drw SF-DRAWABLE the active drawable
;; @param jpg-quality SF-ADJUSTMENT the compression quality desired
;; @param long-side SF-VALUE the long side pixels desired for the final image. If 0, no scaling is applied
(define (webify img drw jpg-quality long-side)
  (let*
    (
       (fname (parse-name (car (gimp-image-get-filename img))))
       (dimensions (get-dimension img long-side))
       (width (car dimensions))
       (height (cadr dimensions))
       (quality (/ jpg-quality 100))
       (scaled (string-append fname "-72.jpg"))
       (num-layers (car (gimp-image-get-layers img)))
       (new-img (car (gimp-image-duplicate img)))
       (drw2 0)
    )
    (begin
      (if (= (string-length fname) 0)
          (gimp-message "Image still not saved. Cannot proceed")
          ;;All ok, let's go on
          (begin
            (gimp-image-undo-group-start new-img)
            (gimp-image-flatten new-img)
            (gimp-image-set-resolution new-img 72 72) ;setting image resolution
            (if (not (= long-side 0)) (gimp-image-scale new-img width height)); scale the image
            (set! drw2 (car (gimp-image-get-active-drawable new-img)))
            (file-jpeg-save 1 new-img drw2 scaled scaled quality 0 0 0 " " 0 1 0 1)
            (gimp-image-undo-group-end new-img)
            (gimp-image-delete new-img)
            )
          )
      )
  )
)

;;;;
;; Take the selected image and convert it into jpg 72dpi
;; 
;; @param path SF-FILENAME the path of the image to process
;; @param jpg-quality SF-ADJUSTMENT the compression quality desired
;; @param long-side SF-VALUE the long side pixels desired for the final image
(define (webify-file path quality long-side)
   (let*
       (
        (image (car (gimp-file-load RUN-NONINTERACTIVE path path))) ;load image
        (drawable (car (gimp-image-get-active-layer image))) ;get drawable
;;        (display (car (gimp-display-new image))) ;display the image
       )
       (webify image drawable quality long-side)
       (gimp-image-delete image)
;;       (gimp-display-delete display)
   )
)

(define (webify-dir dir pattern quality long-side)
;;  (print "inside dir")
  (let*
      (
       (ptr (car (build-pattern dir (car (get-pattern pattern)))))
       (filelist 0)
       (filename 0)
      )
;;    (print ptr)
;;    (gimp-message ptr)
    (set! filelist (cadr (file-glob ptr 1)))
    (while (not (null? filelist))
           (set! filename (car filelist))
;;           (print filename)
           (webify-file filename quality long-side)
           (set! filelist (cdr filelist))
    )
  )
)

(script-fu-register "webify"
                    _"<Image>/Script-Fu/edivad-suite/Webify"
                    "Take the active image and convert it into jpg 72dpi"
                    "Davide Giannella"
                    "(C) Davide Giannella"
                    "October 2008"
                    ""
                    SF-IMAGE "Image" 0
                    SF-DRAWABLE "Drawable" 0
                    SF-ADJUSTMENT "JPG Quality (1 <= quality <= 100)" '(100 1 100 1 10 0 1)
                    SF-VALUE "Long side (px)" "800"
)

(script-fu-register "webify-file"
                    _"<Toolbox>/Xtns/Script-Fu/edivad-suite/Webify/Single File"
                    "Take the selected image and convert it into jpg 72dpi"
                    "Davide Giannella"
                    "(C) Davide Giannella"
                    "October 2008"
                    ""
                    SF-FILENAME "File Path" ""
                    SF-ADJUSTMENT "JPG Quality (1 <= quality <= 100)" '(100 1 100 1 10 0 1)
                    SF-VALUE "Long side (px)" "800"
)

(script-fu-register "webify-dir"
                    _"<Toolbox>/Xtns/Script-Fu/edivad-suite/Webify/Directory"
                    "Take the selected directory and convert all images matching pattern into jpg 72dpi"
                    "Davide Giannella"
                    "(C) Davide Giannella"
                    "October 2008"
                    ""
                    SF-DIRNAME "Directory" ""
                    SF-OPTION "Pattern" PATTERNS
                    SF-ADJUSTMENT "JPG Quality (1 <= quality <= 100)" '(100 1 100 1 10 0 1)
                    SF-VALUE "Long side (px)" "800"
)